<?php

namespace NFT_Marketplace_Core\Backend;


use NFT_Marketplace_Core\Backend\ActDeact\CreateTables;

/**
 * NFT Marketplace Core
 *
 * Builds the table used for showing the Deployed Contracts.
 *
 * @package   NFT Marketplace Core
 * @author    ModelTheme <support@modeltheme.com>
 * @copyright Copyright (C) 2012-2022, Modeltheme, support@modeltheme.com
 * @license   GPL-3.0
 * @link      https://modeltheme.com
 * $END$
 */
class ListTableContract extends ListTableCopy
{

    /**
     * Fallback function for displaying the column data
     *
     * @param $item
     * @param $column_name
     * @return mixed|void
     * @since 1.0.0
     */
    protected function column_default($item, $column_name)
    {
        return $item[$column_name];
    }

    /**
     * Custom display the Listing Price
     *
     * @param $item
     * @return string
     * @since 1.0.0
     */
    public function column_listing_price($item)
    {
        $unit = $item["contract_type"] === "thirdweb" ? "%" : esc_html__(" ether", NFT_MARKETPLACE_CORE_TEXTDOMAIN);
        return $item["listing_price"] . $unit;
    }

    /**
     * @param $item
     * @return mixed
     * @since 1.0.0
     */
    public function column_is_deployed($item)
    {
        if ($item['is_deployed'] === "1") {
            return esc_html__("Deployed", NFT_MARKETPLACE_CORE_TEXTDOMAIN);
        }
        return esc_html__("Not Deployed", NFT_MARKETPLACE_CORE_TEXTDOMAIN);
    }

    /**
     * @param $item
     * @return mixed
     * @since 2.0.0
     */
    public function column_default_contract($item)
    {
        if (is_array($this->items) && count($this->items) === 1) {
            return esc_html__("Yes", NFT_MARKETPLACE_CORE_TEXTDOMAIN);
        }

        if (get_option("nft_marketplace_core_default_contract_blockchain") === false || !isset(get_option("nft_marketplace_core_default_contract_blockchain")[$item['taxonomy_blockchain_id']])) {
            return esc_html__("No default contract set for this blockchain, falling back to the first contract created.", NFT_MARKETPLACE_CORE_TEXTDOMAIN);
        }

        if (get_option("nft_marketplace_core_default_contract_blockchain")[$item['taxonomy_blockchain_id']] === $item["id"]) {
            return esc_html__("Yes", NFT_MARKETPLACE_CORE_TEXTDOMAIN);
        }

        return esc_html__("No", NFT_MARKETPLACE_CORE_TEXTDOMAIN);
    }


    /**
     * @param $item
     * @return mixed
     * @since 1.0.0
     */
    public function column_taxonomy_blockchain_id($item)
    {
        return esc_html(get_term($item['taxonomy_blockchain_id'])->name);
    }

    /**
     * @param $item
     * @return void
     * @since 1.0.0
     */
    public function column_actions($item)
    {
        if ($item["contract_type"] !== "thirdweb") {
            ?>
            <a href="#" data-core-id="nft-marketplace-core-withdraw-contract"
               data-contract-id="<?php echo $item["id"]; ?>" <?php echo nft_marketplace_core_switch_dataset_attribute($item["taxonomy_blockchain_id"]); ?>
               data-contract-address="<?php echo $item['contract_address']; ?>" class="button-primary"
               style="margin-bottom: 10px"><?php esc_html_e("Withdraw", NFT_MARKETPLACE_CORE_TEXTDOMAIN) ?></a>
        <?php }
        if ($item["contract_type"] === "thirdweb") {
            ?>
            <a href="#" data-core-id="nft-marketplace-core-sync-contract-third-web"
               data-contract-id="<?php echo $item["id"]; ?>" <?php echo nft_marketplace_core_switch_dataset_attribute($item["taxonomy_blockchain_id"]); ?>
               data-contract-address="<?php echo $item['contract_address']; ?>"
               class="button-primary"><?php esc_html_e("Sync", NFT_MARKETPLACE_CORE_TEXTDOMAIN) ?></a>
        <?php } else { ?>
        <a href="#" data-core-id="nft-marketplace-core-sync-contract"
           data-contract-id="<?php echo $item["id"]; ?>" <?php echo nft_marketplace_core_switch_dataset_attribute($item["taxonomy_blockchain_id"]); ?>
           data-contract-address="<?php echo $item['contract_address']; ?>"
           class="button-primary"><?php esc_html_e("Sync", NFT_MARKETPLACE_CORE_TEXTDOMAIN) ?></a>
        <?php } ?>

        <a href="<?php echo esc_url(add_query_arg("edit-nft", $item["id"])) ?>"
           class="button-secondary"><?php esc_html_e("Edit", NFT_MARKETPLACE_CORE_TEXTDOMAIN) ?></a>

        <?php
        if (!(is_array($this->items) && count($this->items) === 1) || is_bool(get_option("nft_marketplace_core_default_contract_blockchain")) || get_option("nft_marketplace_core_default_contract_blockchain")[$item['taxonomy_blockchain_id']] !== $item["id"]) {
            ?>
            <a href="<?php echo esc_url(add_query_arg("make-default", $item["id"])) ?>"
               class="button-secondary"><?php esc_html_e("Make Default", NFT_MARKETPLACE_CORE_TEXTDOMAIN) ?></a>
            <?php
        }
    }

    /**
     * @param $item
     * @return void
     * @since 1.0.0
     */
    public function column_delete($item)
    {
        echo '<a href="#" class="button-primary" data-term-id="' . $item["taxonomy_blockchain_id"] . '"  id="nft-marketplace-core-delete-contract" style="background: red;">' . esc_html__("Delete", NFT_MARKETPLACE_CORE_TEXTDOMAIN) . '</a>';
    }


    /**
     * @return void
     * @since 1.0.0
     */
    public function prepare_items()
    {
        global $wpdb;
        $posts_per_page = 50;
        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();
        $data = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "nft_marketplace_core_contracts LIMIT " . $posts_per_page, ARRAY_A);
        //usort( $data, array( &$this, 'sort_data' ) );

        $currentPage = $this->get_pagenum();
        $totalItems = count($data);

        $this->set_pagination_args(array(
            'total_items' => $totalItems,
            'per_page' => $posts_per_page
        ));

        $data = array_slice($data, (($currentPage - 1) * $posts_per_page), $posts_per_page);

        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->items = $data;
    }

    /**
     * Gets a list of columns.
     *
     * The format is:
     * - `'internal-name' => 'Title'`
     *
     * @return array
     * @since 1.0.0
     * @abstract
     *
     */
    public function get_columns()
    {
        return [
            "id" => esc_html__("ID", NFT_MARKETPLACE_CORE_TEXTDOMAIN),
            "taxonomy_blockchain_id" => esc_html__("Blockchain", NFT_MARKETPLACE_CORE_TEXTDOMAIN),
            "contract_address" => esc_html__("Contract Address", NFT_MARKETPLACE_CORE_TEXTDOMAIN),
            "contract_owner" => esc_html__("Recipient Address", NFT_MARKETPLACE_CORE_TEXTDOMAIN),
            "listing_price" => esc_html__("Platform Fees", NFT_MARKETPLACE_CORE_TEXTDOMAIN),
            "is_deployed" => esc_html__("Is Deployed", NFT_MARKETPLACE_CORE_TEXTDOMAIN),
            "version" => esc_html__("Version", NFT_MARKETPLACE_CORE_TEXTDOMAIN),
            "default_contract" => esc_html__("Default Contract", NFT_MARKETPLACE_CORE_TEXTDOMAIN),
            "actions" => esc_html__("Actions", NFT_MARKETPLACE_CORE_TEXTDOMAIN),
            "delete" => esc_html__("Delete", NFT_MARKETPLACE_CORE_TEXTDOMAIN),
        ];
    }

    /**
     * @return void
     * @since 1.0.0
     */
    private function table_data()
    {
    }

    /**
     * @return array
     * @since 1.0.0
     */
    private function get_hidden_columns()
    {
        return [];
    }
}
