<?php

namespace NFT_Marketplace_Core\Backend;

use NFT_Marketplace_Core\Backend\ActDeact\CreatePages;
use NFT_Marketplace_Core\Internals\Metas\BlockchainMeta;
use WPConfigTransformer;
use WPDesk\Notice\Notice;

class Tools
{
    /**
     * Handle all forms on ?post_type=nft-listing&page=nft-marketplace-core-panel-tools
     *
     * @return void
     * @since 3.2.0
     */
    public function handleForms() {
        if (isset($_POST["mk_pages"])) {
            $this->createPages();
        } elseif(isset($_POST["mk_config_plugin"])) {
            $this->configurePlugins();
        } elseif(isset($_POST["mk_import_blockchain"])) {
            if (!is_numeric($_POST["mk_selected_option"])) die();
            $this->importBlockchain($_POST["mk_selected_option"], $_POST["mk_all_values"]);
        } elseif (isset($_POST["mk_reset_plugin"])) {
            $this->resetPlugin();
        } elseif (isset($_POST["mk_turn_debug_on"])) {
            $this->toggleDebug();
        }
    }

    /**
     * Create all pages required for Enefti - NFT Marketplace Core
     *
     * @return Notice
     * @since 3.2.0
     */
    protected function createPages() {
        $cp = new CreatePages();

        if ($cp->run()) {
            return \wpdesk_wp_notice(esc_html__("Pages created successfully!", NFT_MARKETPLACE_CORE_TEXTDOMAIN),"success", true);
        }
        return \wpdesk_wp_notice(esc_html__("Some pages could not be created! Maybe they already exist, check and then come back.", NFT_MARKETPLACE_CORE_TEXTDOMAIN),"error", true);
    }

    /**
     * Configure all the plugins necessary for Enefti - NFT Marketplace Core
     *
     * @return Notice
     * @since 3.2.0
     */
    protected function configurePlugins() {
        //WPNC
        $options = get_option("wpnc");

        $options["isEventAfterCreationActive"] = true;
        $options["isEventBeforeCreationActive"] = true;

        if (update_option("wpnc", $options)) {
            return \wpdesk_wp_notice(esc_html__("WordPress NFT Creator configured successfully!", NFT_MARKETPLACE_CORE_TEXTDOMAIN),"success", true);
        }

        if (is_plugin_active("wpnbm/wpnbm.php")) {
            $options = get_option("wpnbm");

            $options["isEventAfterCreationActive"] = true;

            if (update_option("wpnbm", $options)) {
                return \wpdesk_wp_notice(esc_html__("WordPress NFT Bulk Minting configured successfully!", NFT_MARKETPLACE_CORE_TEXTDOMAIN),"success", true);
            }
        }

        return \wpdesk_wp_notice(esc_html__("Could not configure plugins.", NFT_MARKETPLACE_CORE_TEXTDOMAIN),"error", true);
    }

    /**
     * Configure all the plugins necessary for Enefti - NFT Marketplace Core
     *
     * @return Notice
     * @since 3.2.0
     */
    protected function importBlockchain($selectedBlockchain, $list) {
        $list = json_decode(stripslashes($list));

        $blockchain = $list[$selectedBlockchain];
        $term = wp_insert_term($blockchain->name, "nft_listing_blockchains", );

        if (is_wp_error($term)) return \wpdesk_wp_notice(esc_html__("Could not add blockchain! ". $term->get_error_message(), NFT_MARKETPLACE_CORE_TEXTDOMAIN),"error", true);
        $termID = $term["term_id"];

        add_term_meta($termID, BlockchainMeta::rpc, $blockchain->rpc[0]);
        add_term_meta($termID, BlockchainMeta::chainid, '0x' . dechex($blockchain->chainId));
        add_term_meta($termID, BlockchainMeta::symbol, $blockchain->nativeCurrency->symbol);

        return \wpdesk_wp_notice(esc_html__("Blockchain $blockchain->name added successfully!", NFT_MARKETPLACE_CORE_TEXTDOMAIN),"success", true);
    }

    /**
     * Resets the plugin as it was never installed (keeps the tables, but the date is deleted)
     *
     * @return Notice
     * @since 3.2.0
     */
    protected function resetPlugin() {

        global $wpdb;
        $prefix = $wpdb->prefix;

        // Delete Custom Tables
        $wpdb->query( "DROP TABLE IF EXISTS ${prefix}nft_marketplace_core_contracts" );
        $wpdb->query( "DROP TABLE IF EXISTS ${prefix}nft_marketplace_core_logs_values" );
        $wpdb->query( "DROP TABLE IF EXISTS ${prefix}nft_marketplace_core_logs" );
        $wpdb->query( "DROP TABLE IF EXISTS ${prefix}nft_marketplace_core_offers" );
        $wpdb->query( "DROP TABLE IF EXISTS ${prefix}nft_marketplace_core_transactions" );

        // Delete All NFTs
        $wpdb->query("DELETE FROM ${prefix}posts WHERE post_type = 'nft-listing'");
        $wpdb->query("DELETE FROM ${prefix}postmeta WHERE meta_key LIKE '%nft_marketplace_core%' ");

        // Delete Taxonomies, currencies/categories/collections/blockchains/etc.
        $wpdb->query("DELETE FROM ${prefix}term_taxonomy WHERE taxonomy IN ('nft_listing_blockchains', 'nft-listing-collection', 'nft-listing-currency', 'nft-listing-category') ");
        $wpdb->query("DELETE FROM ${prefix}termmeta WHERE meta_key LIKE '%nft_marketplace_core%' ");

        // Delete Pages (from settings and default)


        // Delete settings
        $wpdb->query("DELETE FROM ${prefix}options WHERE option_name LIKE '%nft_marketplace_core%'");

        // Remove role
        remove_role( "nft_vendor" );

        // Deactivate the plugin. and redirect to first page.
        deactivate_plugins( "nft-marketplace-core/nft-marketplace-core.php" );

        echo '<meta http-equiv="refresh" content="0; url='.get_admin_url().'" />';

        return \wpdesk_wp_notice(esc_html__("Plugin reset successfully! The page will reload automatically.", NFT_MARKETPLACE_CORE_TEXTDOMAIN),"success", true);
    }

    /**
     * Toggle WP_DEBUG.
     *
     * @return Notice
     * @since 3.2.0
     */
    private function toggleDebug()
    {
        $nextStatus = WP_DEBUG ? "off" : "on";
        try {
            $config_transformer = new WPConfigTransformer( ABSPATH.'wp-config.php' );

            $config_transformer->update( 'constant', 'WP_DEBUG', (string) !WP_DEBUG, array( ) );
            $config_transformer->update( 'constant', 'WP_DEBUG_LOG', (string) !WP_DEBUG, array() );
            $config_transformer->update( 'constant', 'WP_DEBUG_DISPLAY', (string) !WP_DEBUG, array( ) );

        } catch (\Exception $e) {
            return \wpdesk_wp_notice(esc_html__("Could not toggle debug!", NFT_MARKETPLACE_CORE_TEXTDOMAIN),"error", true);
        }

        return \wpdesk_wp_notice(esc_html__("Debug turned $nextStatus.", NFT_MARKETPLACE_CORE_TEXTDOMAIN),"success", true);
    }
}
