<?php

namespace NFT_Marketplace_Core\Integrations;

use NFT_Marketplace_Core\Engine\Base;
use NFT_Marketplace_Core\Internals\Helpers\Marketplace;
use NFT_Marketplace_Core\Internals\Metas\BlockchainMeta;

/**
 * Handles integration with the NFT Bulk Minting.
 *
 * @package   NFT Marketplace Core
 * @author    ModelTheme <support@modeltheme.com>
 * @copyright Copyright (C) 2012-2022, ModelTheme, support@modeltheme.com
 * @license   GPL-3.0
 * @link      https://modeltheme.com
 */
class BulkMinting extends Base
{
    /**
     * Initialize the class.
     *
     * @return void
     */
    public function initialize()
    {
        parent::initialize();

        add_filter('wpnbm_event_after_minting_filter', [$this, 'importNFTs']);
    }


    /**
     *
     * Handles importing NFTs from NFT Bulk Minting
     *
     * @param \WP_REST_Request $request
     * @return \WP_REST_Response
     * @since 3.2.0
     */
    public function importNFTs($request)
    {
        $transaction = $request->get_param("transaction");
        if (!$transaction) {
            return new \WP_REST_Response(["success" => true], 200);
        }
        $termId = get_terms([
            'taxonomy' => "nft_listing_blockchains",
            'hide_empty' => false,
            'meta_query' => array(
                array(
                    'key' => BlockchainMeta::chainid,
                    'value' => "0x" . dechex($transaction["chainId"]),
                    'compare' => '='
                )
            )
        ]);

        $formData = $request->get_param("formData");
		if ($formData["contractType"] === "Existing") {
			$collection = get_terms(
				array(
					"taxonomy" => "nft-listing-collection",
					"hide_empty" => false,
					"meta_query" => array(
						array(
							"key" => "nft_marketplace_core_taxonomy_collection_address",
							"value" => $formData["contract"]
						)
					)
				)
			)[0];
			$nfts = get_term_meta($collection->term_id, "nft_marketplace_core_taxonomy_collection_nfts", true);
			update_term_meta($collection->term_id, "nft_marketplace_core_taxonomy_collection_nfts", (int)$nfts + (int)$request->get_param("mintedNFTs"));
			$collectionId = $collection->term_id;
		} else if ($formData["contractType"] === "New") {
			// create collection
			// @todo add collection to post if it does exist already, take into consideration that the collection might be created by another user
			$collection = wp_insert_term($formData["title"], "nft-listing-collection",
				array(
					'description' => $formData["desc"],
				)
			);
			add_term_meta($collection["term_id"], "nft_marketplace_core_taxonomy_collection_address", $request->get_param("address"));
			add_term_meta($collection["term_id"], "nft_marketplace_core_taxonomy_collection_type", "thirdweb");
			add_term_meta($collection["term_id"], "nft_marketplace_core_taxonomy_collection_nfts", (int)$request->get_param("mintedNFTs"));
			$collectionId = $collection["term_id"];
		}

        if (is_wp_error($collection)) {
            return new \WP_REST_Response(["success" => false, "message" => esc_html__("Collection with the same name already exists, we will import your NFTs without a collection assigned.", NFT_MARKETPLACE_CORE_TEXTDOMAIN)], 400);
        }

		try {
			Marketplace::queueForImport(
				$transaction["to"],
				(int)$request->get_param("mintedNFTs"),
				get_current_user_id(),
				$formData["title"],
				$termId[0]->term_id,
				is_wp_error($collection) ? null : $collectionId,
				null,
				"bulk",
				$formData["contractType"] === "Existing"
			);
		} catch (\Exception $e) {
			return new \WP_REST_Response(["success" => false, "me"], 200);
		}

		return new \WP_REST_Response(["success" => true], 200);
    }
}
