<?php
namespace NFT_Marketplace_Core\Integrations;

use NFT_Marketplace_Core\Engine\Base;
use NFT_Marketplace_Core\Internals\Helpers\Marketplace;
use NFT_Marketplace_Core\Internals\Metas\BlockchainMeta;

/**
 * Handles integration with the NFT Creator.
 *
 * @package   NFT Marketplace Core
 * @author    ModelTheme <support@modeltheme.com>
 * @copyright Copyright (C) 2012-2022, ModelTheme, support@modeltheme.com
 * @license   GPL-3.0
 * @link      https://modeltheme.com
 */
class Creator extends Base
{
    /**
     * Initialize the class.
     *
     * @return void
     */
	public function initialize()
	{
		parent::initialize();

		add_filter('wpnc_event_after_creation_filter', [$this, 'add_nft_from_creator']);
		add_filter('wpnc_event_before_creation_filter', [$this, 'validate_from_creator']);
		add_filter('wpnc_marketplace_contracts', [$this, 'provideContracts']);
		add_filter('wpnc_marketplace_currencies', [$this, 'provideCurrencies']);
	}

	public function provideCurrencies($contracts) {
		$contracts = [];
		$terms = get_terms( array(
			'taxonomy'   => 'nft-listing-currency',
			'hide_empty' => false,
		) );
		foreach ($terms as $item) {
			$contracts[] = [
				"chainId" => hexdec( get_term_meta(get_term_meta($item->term_id, "nft_marketplace_core_taxonomy_currency_blockchain", true), "nft_marketplace_core_taxonomy_blockchain_currency_chainid", true)),
				"networkName" => get_term_meta($item->term_id, "nft_marketplace_core_taxonomy_currency_symbol", true),
                "address" => get_term_meta($item->term_id, "nft_marketplace_core_taxonomy_currency_contract", true)
			];
		}
		return $contracts;
	}

	public function provideContracts($contracts) {
		global $wpdb;
		$contracts = [];

		$contractsFromDb = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "nft_marketplace_core_contracts");
		foreach ($contractsFromDb as $item) {
			$contracts[] = [
				"chainId" => hexdec(get_term_meta($item->taxonomy_blockchain_id, "nft_marketplace_core_taxonomy_blockchain_currency_chainid", true)),
				"address"=> $item->contract_address
			];
		}
		return $contracts;
	}

    /**
     * Before publishing nft to blockchain.
     * @param $request \WP_REST_Request
     * @return \WP_REST_Response
     */
    public function validate_from_creator(\WP_REST_Request $request): \WP_REST_Response
    {
        $data = $request->get_param("formData");

        $required = array('levels', 'name', 'properties', 'stats');

        if (count(array_intersect_key(array_flip($required), $data)) !== count($required)) {
            return new \WP_REST_Response(["success" => false, "message" => esc_html__("Core: Something went wrong!", NFT_MARKETPLACE_CORE_TEXTDOMAIN)], 400);
        }

        if (!isset($data["imageLink"])) {
            if (!is_array($data["image"])) {
                return new \WP_REST_Response(["success" => false, "message" => esc_html__("Core: Not a valid image!", NFT_MARKETPLACE_CORE_TEXTDOMAIN)], 400);
            }
        }

        return new \WP_REST_Response(["success" => true, "message" => esc_html__("Core: Everything went alright!", NFT_MARKETPLACE_CORE_TEXTDOMAIN)], 201);
    }

    /**
     * After publishing nft to blockchain.
     * @param $request \WP_REST_Request
     * @return \WP_REST_Response
     * @throws \Exception
     */
    public function add_nft_from_creator(\WP_REST_Request $request): \WP_REST_Response
    {
        $data = $request->get_param("formData");
        $listingId = $request->get_param("listingId");

        $vr = $this->validate_from_creator($request);
        if ($vr->get_data()["success"] === false) {
            return $vr;
        }
		$transaction = $request->get_param("transaction");

		$args = array(
            'hide_empty' => false, // also retrieve terms which are not used yet
            'meta_query' => array(
                array(
                    'key' => 'nft_marketplace_core_taxonomy_blockchain_currency_chainid',
                    'value' => '0x' . dechex($transaction["chainId"]),
                )
            ),
            'taxonomy' => 'nft_listing_blockchains',
        );

        $term = get_terms($args);

        if (is_wp_error($term)) {
            return new \WP_REST_Response(["success" => false, "payload" => [
                "message" => esc_html__("Blockchain ${transaction["chainId"]} not supported!", NFT_MARKETPLACE_CORE_TEXTDOMAIN)
            ]], 400);
        }

        $term = $term[0];

		$posts = Marketplace::queueForImport(
			$transaction["to"],
			1,
			get_current_user_id(),
			$data["name"],
			$term->term_id,
			null,
			is_null($listingId) ? null : [0 => $listingId]
		);

		return new \WP_REST_Response(["success" => true, "payload" => [
			"image" => "",
			"permalink" => get_permalink($posts[0]),
		]], 201);
    }
}
