<?php

namespace NFT_Marketplace_Core\Integrations\Widgets;

use WPH_Widget;

/**
 * Widget for filtering NFTs by their terms.
 *
 * @package   NFT Marketplace Core
 * @author    ModelTheme <support@modeltheme.com>
 * @copyright All rights reserved for ModelTheme
 * @license   GPL-3.0
 * @link      https://modeltheme.com
 */
class TermFilterWidget extends WPH_Widget
{
	/**
	 * Initialize the widget
	 *
	 * @return void
	 */
	public function __construct()
	{ // phpcs:ignore
		$args = array(
			'label' => \__('NFT: Taxonomy Filter', NFT_MARKETPLACE_CORE_TEXTDOMAIN),
			'description' => \__('Filter NFTs by the chosen taxonomy', NFT_MARKETPLACE_CORE_TEXTDOMAIN),
			'slug' => 'nft-marketplace-core-widget-taxonomy-filter',
			'options' => array('cache' => true)
		);

		$args['fields'] = array(
			// Title field
			array(
				// Field name/label
				'name' => \__('Title', NFT_MARKETPLACE_CORE_TEXTDOMAIN),
				// Field description
				'desc' => \__('Enter the widget title.', NFT_MARKETPLACE_CORE_TEXTDOMAIN),
				// Field id
				'id' => 'title',
				// Field type ( text, checkbox, textarea, select, select-group, taxonomy, taxonomyterm, pages, hidden )
				'type' => 'text',
				// Class, rows, cols
				'class' => 'widefat',
				// Default value
				'std' => \__('Blockchains', NFT_MARKETPLACE_CORE_TEXTDOMAIN),
				/**
				 * Set the field validation type/s
				 * 'alpha_dash'
				 * Returns FALSE if the value contains anything other than alpha-numeric characters, underscores or dashes.
				 * 'alpha'
				 * Returns FALSE if the value contains anything other than alphabetical characters.
				 * 'alpha_numeric'
				 * Returns FALSE if the value contains anything other than alpha-numeric characters.
				 * 'numeric'
				 * Returns FALSE if the value contains anything other than numeric characters.
				 * 'boolean'
				 * Returns FALSE if the value contains anything other than a boolean value ( true or false ).
				 * You can define custom validation methods. Make sure to return a boolean ( TRUE/FALSE ).
				 * Example:
				 * 'validate' => 'my_custom_validation',
				 * Will call for: $this->my_custom_validation( $value_to_validate );
				 */
				'validate' => 'alpha_dash',
				/**
				 * Filter data before entering the DB
				 * strip_tags ( default )
				 * wp_strip_all_tags
				 * esc_attr
				 * esc_url
				 * esc_textarea
				 */
				'filter' => 'strip_tags|esc_attr',
			),
			// Taxonomy Field
			array(
				'name' => \__('Taxonomy', NFT_MARKETPLACE_CORE_TEXTDOMAIN),
				'desc' => \__('Set the taxonomy.', NFT_MARKETPLACE_CORE_TEXTDOMAIN),
				'id' => 'taxonomy',
				'type' => 'taxonomy',
				'class' => 'widefat',
			),
//            array(
//                'name' => \__('Hide empty', NFT_MARKETPLACE_CORE_TEXTDOMAIN),
//                'desc' => \__('Hide taxonomies without any content.', NFT_MARKETPLACE_CORE_TEXTDOMAIN),
//                'id' => 'empty-taxonomy',
//                'type' => 'checkbox',
//                'class' => 'widefat',
//            ),
		);
		// Create widget
		$this->create_widget($args);
	}

	/**
	 * Custom validation for this widget
	 *
	 * @param string $value The text.
	 * @return bool
	 * @since {{plugin_version}}
	 */
	public function my_custom_validation(string $value)
	{
		return \strlen($value) <= 1;
	}

	/**
	 * Output function
	 *
	 * @param array $args The argument shared to the output from WordPress.
	 * @param array $instance The settings saved.
	 * @return void
	 * @since {{plugin_version}}
	 */
	public function widget($args, $instance)
	{ //phpcs:ignore
		echo $args['before_widget'];
		?>
		<div class="faq-drawer">
			<input id="faq-drawer-<?php esc_attr_e($args["widget_id"]); ?>"
				   class="faq-drawer__trigger thoidinput widget" type="checkbox"/>
			<label class="faq-drawer__title" for="faq-drawer-<?php esc_attr_e($args["widget_id"]); ?>">
				<?php
				echo $args['before_title'] . $instance["title"] . $args['after_title'];
				?>
			</label>
			<div class="faq-drawer__content-wrapper">
				<div class="faq-drawer__content">
					<?php
					// Display the price filter form
					$this->display_price_filter_form($instance);
					?>
				</div>
			</div>
		</div>
		<?php
		echo $args['after_widget'];
	}

	function filterTermsByCpt($taxonomy, $post_types = array())
	{
		global $wpdb;

		$post_types = (array)$post_types;
		$key = 'wpse_terms' . md5($taxonomy . serialize($post_types));
		$results = wp_cache_get($key);

		if (false === $results) {
			$where = " WHERE 1=1";
			if (!empty($post_types)) {
				$post_types_str = implode(',', $post_types);
				$where .= $wpdb->prepare(" AND p.post_type IN(%s)", $post_types_str);
			}

			$where .= $wpdb->prepare(" AND tt.taxonomy = %s", $taxonomy);

			$query = "
              SELECT t.*, COUNT(*)
              FROM $wpdb->terms AS t
              INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id
              INNER JOIN $wpdb->term_relationships AS r ON r.term_taxonomy_id = tt.term_taxonomy_id
              INNER JOIN $wpdb->posts AS p ON p.ID = r.object_id
              $where
              GROUP BY t.term_id
            ";

			$results = $wpdb->get_results($query);
			wp_cache_set($key, $results);
		}

		return $results;
	}

	/**
	 * Initialize the class.
	 *
	 * @return void
	 */
	public function initialize()
	{
		\add_action(
			'widgets_init',
			static function () {
				\register_widget('NFT_Marketplace_Core\Integrations\Widgets\TermFilterWidget');
			}
		);
	}

	private function display_price_filter_form($instance)
	{
		$out = "";
		$terms = $this->filterTermsByCpt($instance["taxonomy"], ['nft-listing']);
		if (is_array($terms) && count($terms) === 0) {
			$out .= "<p>" . esc_html__("Nothing found.", NFT_MARKETPLACE_CORE_TEXTDOMAIN) . "</p>";
		}

		/** @var \WP_Term $term */
		foreach ($terms as $term) {
			$term_count = new \WP_Query(array(
				'post_type' => 'nft-listing',
				'tax_query' => array(
					array(
						'taxonomy' => $instance["taxonomy"],
						'field' => 'slug',
						'terms' => $term->slug,
					)
				)
			));
			$count = $term_count->found_posts;
			$selected = isset($_GET['taxonomy_' . $instance["taxonomy"]]) && in_array($term->slug, $_GET['taxonomy_' . $instance["taxonomy"]]) ? "checked" : "";
			$out .= '
				<div style="display: flex;align-items: center;gap: 10px">
					<input id="check-' . $term->slug . '"  ' . $selected . ' type="checkbox" name="taxonomy_' . $instance["taxonomy"] . '[]" value="' . $term->slug . '" type="checkbox" style="display: none;">
					<label for="check-' . $term->slug . '" class="check">
						<svg width="18px" height="18px" viewBox="0 0 18 18">
							<path d="M1,9 L1,3.5 C1,2 2,1 3.5,1 L14.5,1 C16,1 17,2 17,3.5 L17,14.5 C17,16 16,17 14.5,17 L3.5,17 C2,17 1,16 1,14.5 L1,9 Z"></path>
							<polyline points="1 9 7 14 15 4"></polyline>
						</svg>
						<span style="margin-left: 10px">' . $term->name . ' (' . $count . ')</span>
					</label>
				</div>';
		}

		echo $out; // phpcs:ignore
	}
}
