<?php

namespace NFT_Marketplace_Core\Internals\Helpers;



class Assets
{
	/**
	 * Get currency image as HTML for the specified term id.
	 * @param $termId
	 * @param array $customSizes
	 * @return false|string
	 * @since 3.0.1
	 */
	public static function getCurrencyImageSrcByTermId($termId, array $customSizes = [30, 30])
	{
		$nft_marketplace_core_nft_listing_blockchains_term_meta = get_term_meta($termId);

		if (isset($nft_marketplace_core_nft_listing_blockchains_term_meta["nft_marketplace_core_taxonomy_blockchain_currency_image"])) {
			$image = wp_get_attachment_image_src($nft_marketplace_core_nft_listing_blockchains_term_meta["nft_marketplace_core_taxonomy_blockchain_currency_image"][0], $customSizes, true);
			if (is_array($image)) {
				return "<img src='" . $image[0] . "' width='" . $customSizes[0] . "' height='" . $customSizes[1] . "' alt='" . esc_attr__("Blockchain Icon", NFT_MARKETPLACE_CORE_TEXTDOMAIN) . "'/>";
			}
			return $image;
		} elseif (isset($nft_marketplace_core_nft_listing_blockchains_term_meta["nft_marketplace_core_taxonomy_currency_symbol"]) && $nft_marketplace_core_nft_listing_blockchains_term_meta["nft_marketplace_core_taxonomy_currency_symbol"]) {
			$image = wp_get_attachment_image_src($nft_marketplace_core_nft_listing_blockchains_term_meta["currency-image-id"][0], $customSizes, true);
			if (is_array($image)) {
				return "<img src='" . $image[0] . "' width='" . $customSizes[0] . "' height='" . $customSizes[1] . "' alt='" . esc_attr__("Token Icon", NFT_MARKETPLACE_CORE_TEXTDOMAIN) . "'/>";
			}
			return $image;
		}

		return self::getDefaultThumbnailSrc(["image" => "width:30px;height:30px;"]);
	}

	/**
	 * Get the NFT Display asset's final url wrapped in html.
	 * @param $postid
	 * @param string $size
	 * @param bool $showExplicit
	 * @return mixed|string
	 * @since 3.0.1
	 */
	public static function getNFTDisplayAsset($postid, $size = "large", bool $showExplicit = true)
	{
		$isExplicit = get_post_meta($postid, "nft_marketplace_core_nft_listing_explicit_meta", true);

		if ($isExplicit) {
			return self::getNFTDisplayAssetExplicit($postid, $size, $showExplicit);
		}

		return self::getNFTDisplayAssetRAW($postid, $size);
	}

	/**
	 * Wrap the NFT inside an explicit tag.
	 * @param $postid
	 * @param string $size
	 * @param bool $showExplicitText
	 * @return string
	 * @since 3.0.1
	 */
	private static function getNFTDisplayAssetExplicit($postid, $size = "large", bool $showExplicitText = true): string
	{
		$largeSize = "";

		if ($showExplicitText) {
			$largeSize = "
            <div class='overlay'>
                <h5>" . esc_html__('This content might be disturbing. Proceed with caution!', NFT_MARKETPLACE_CORE_TEXTDOMAIN) . "</h5>
                <button id='nft-marketplace-core-nsfw-remove-button'>" . esc_html__('View Regardless', NFT_MARKETPLACE_CORE_TEXTDOMAIN) . "</button>
            </div>";
		}

		return "
        <div id='nft-marketplace-core-hide-content-nsfw' class='nft-marketplace-core-hide-content-nsfw'>
                " . self::getNFTDisplayAssetRAW($postid, $size) . "
            $largeSize
        </div>
        ";
	}

	/**
	 * Get the NFT Display asset's url.
	 * @param $postid
	 * @param string $size
	 * @return mixed|string
	 * @since 3.0.1
	 */
	private static function getNFTDisplayAssetRAW($postid, $size = "large")
	{

		$sizeT = [
			"image" => "width:360px;height:360px;object-fit: scale-down;",
			"default" => [360, 360]
		];

		if ($size == 'thumbnail') {
			$sizeT = [
				"image" => "width: 80px; height: 80px;object-fit: scale-down;",
			];
		} elseif ($size == 'medium') {
			$sizeT = [
				"image" => "width: 100%; height: 100%;object-fit: cover;",
			];
		}

		$external = get_post_meta($postid, "nft_marketplace_core_nft_listing_external_image", true);
		$assets = get_post_meta($postid, "nft_marketplace_core_nft_listing_assets", true);

//        if ($external === "" && (!is_array($assets) || !isset($assets[0]))) {
//            return esc_html__("Could not determine the location of the assets.",NFT_MARKETPLACE_CORE_TEXTDOMAIN);
//        }

		if (is_array($assets) && strpos(wp_check_filetype(wp_get_attachment_url($assets[0]))["type"], 'image/') !== 0 && !function_exists("nft_marketplace_core_media_types_addon_media")) {
			return esc_html__("You need to have the Media Types for NFT Marketplace Core Plugin to display assets of this type.", NFT_MARKETPLACE_CORE_TEXTDOMAIN);
		}
		if ($external !== "") {
			$filter = apply_filters("nft_marketplace_core_process_display_asset", false, $postid, $external, $size);

			if ($filter) {
				return $filter;
			}
		}


		if (!is_array($external) && $external != "") {
			return '<img  src="' . Assets::ipfsToURL($external) . '"   style="' . $sizeT['image'] . '"  alt=""/>';
		}

		if (is_array($external) && isset($external[0])) {
			return '<img  src="' . Assets::ipfsToURL($external[0]) . '"   style="' . $sizeT['image'] . '"  alt=""/>';
		}

		$thumbnail = wp_get_attachment_image(get_post_thumbnail_id($postid), $size);

		if ($thumbnail) {
			return $thumbnail;
		}

		return self::getDefaultThumbnailSrc($sizeT);
	}

	/**
	 * Get the default thumbnail as a HTML element.
	 * @param array $sizeT
	 * @return string
	 * @since 3.0.1
	 */
	public static function getDefaultThumbnailSrc(array $sizeT = ["image" => ""]): string
	{
		$defaultThumbnailUrl = self::getPlaceholderLocation();
		$size = $sizeT["image"];
		$alt = esc_html__("No image available", NFT_MARKETPLACE_CORE_TEXTDOMAIN);

		return "<img src='$defaultThumbnailUrl' style='$size' alt='$alt'/>";
	}

	/**
	 * Get placeholder location.
	 * @return string
	 * @since 3.0.1
	 */
	public static function getPlaceholderLocation(): string
	{
		return plugin_dir_url(dirname(__FILE__, 2)) . "assets/images/placeholder.png";
	}

	/**
	 * Transform ipfs links to gateway links.
	 * @param $link
	 * @return array|string|string[]
	 * @since 3.0.1
	 */
	public static function ipfsToURL($link)
	{
		if (strpos($link, "ipfs://") !== 0) return $link;

		$ipfsLink = 'https://gateway.ipfs.io/ipfs/${hash}'; // default
		if (isset(get_option('nft_marketplace_core_panel_extras_tab')['nft_marketplace_core_panel_extras_tab_ipfs_providers'])) {
			foreach (get_option('nft_marketplace_core_panel_extras_tab')['nft_marketplace_core_panel_extras_tab_ipfs_providers'] as $ipfsProvider) {
				$asLink = str_replace('${hash}', str_replace("ipfs://", "", $link), $ipfsProvider);
				if ($socket = @ fsockopen($asLink, 80, $errno, $errstr, 5)) {
					$ipfsLink = $ipfsProvider;
					break;
				}
			}
		}

		return str_replace('${hash}', str_replace("ipfs://", "", $link), $ipfsLink);
	}
}
