<?php

namespace NFT_Marketplace_Core\Internals\Helpers;

/**
 * NFT Marketplace Core
 *
 * Build the dataset attribute for the most common functions
 *
 * @package   NFT Marketplace Core
 * @author    ModelTheme <support@modeltheme.com>
 * @copyright All rights reserved for ModelTheme
 * @license   GPL-3.0
 * @link      https://modeltheme.com
 */
class BlockchainDatasetBuilder
{

    /**
     *
     * Required data to list a NFT for sale.
     *
     * @param $tokenId
     * @param $nftAddress
     * @param $marketplaceAddress
     * @param $termId
     * @param string $id
     * @return string
     * @since 3.1.0
     */
    public static function listForSale($tokenId, $nftAddress, $marketplaceAddress, $termId, $id = "nft-marketplace-core-list-for-sale")
    {
        return self::build(
            self::id($id),
            self::tokenId($tokenId),
            self::nftAddress($nftAddress),
            self::marketplaceAddress($marketplaceAddress),
            self::switchNetwork($termId)
        );
    }

	/**
	 *
	 * Required data for purchasing a listing.
	 *
	 * @param $listingId
	 * @param $marketplaceAddress
	 * @param $termId
	 * @param string $id
	 * @return string
	 * @since 3.1.0
	 */
    public static function purchaseListing($listingId, $marketplaceAddress, $termId, $id = "nft-marketplace-core-purchase-nft-form")
    {
        return self::build(
            self::id($id),
            self::listingId($listingId),
            self::marketplaceAddress($marketplaceAddress),
			self::switchNetwork($termId),
		);
    }

    /**
     *
     * Required data for canceling a listing.
     *
     * @param $listingId
     * @param $marketplaceAddress
     * @param string $id
     * @return string
     * @since 3.1.0
     */
    public static function cancelListing($listingId, $marketplaceAddress, $termId, $id = "nft-marketplace-core-cancel-listing")
    {
        return self::build(
            self::id($id),
            self::listingId($listingId),
            self::marketplaceAddress($marketplaceAddress),
			self::switchNetwork($termId),
		);
    }

    /**
     *
     * Required data for refreshing metadata.
     *
     * @param $postID
     * @param $tokenId
     * @param string $id
     * @return string
     * @since 3.1.0
     */
    public static function refreshMetadata($postID, $tokenId, $id = "nft-marketplace-core-refresh-metadata")
    {
        return self::build(
            self::id($id),
            self::postId($postID),
            self::tokenId($tokenId),
        );
    }

    /**
     *
     * Required data for making an offer.
     *
     * @param $chainId
     * @param $listingId
     * @param $quantity
     * @param $currencyAddress
     * @param $marketplaceAddress
     * @param $termId
     * @param string $id
     * @return string
     * @since 3.1.0
     */
    public static function makeOffer($chainId, $tokenId, $nftAddress, $quantity, $currencyAddress, $marketplaceAddress, $termId, $id = "nft-marketplace-core-make-offer")
    {
        return self::build(
            self::id($id),
            self::chainId($chainId),
			self::tokenId($tokenId),
			self::nftAddress($nftAddress),
            self::quantity($quantity),
            self::currencyAddress($currencyAddress),
            self::marketplaceAddress($marketplaceAddress),
			self::switchNetwork($termId),
        );
    }

    /**
     *
     * Internal function used to chain the attributes.
     *
     * @param ...$args string list of attributes made with the functions below
     * @return string
     * @since 3.1.0
     */
    private static function build(...$args)
    {
        return implode(" ", $args);
    }

    private static function id($id)
    {
        return "id=\"" . $id . "\"";
    }

    private static function tokenId($id)
    {
        return "data-tokenid=\"" . esc_attr($id) . "\"";
    }

    private static function marketplaceAddress($address)
    {
        return "data-market-address=\"" . esc_attr($address) . "\"";
    }

    private static function nftAddress($address)
    {
        return "data-address=\"" . esc_attr($address) . "\"";
    }

    /**
     *
     * Builds the data required to change the network.
     *
     * @TODO change to private modifier in the next versions.
     *
     * @param $termid
     * @return string
     * @since 3.1.0
     */
    public static function switchNetwork($termid)
    {
        return 'data-force-switch-to="' . esc_attr(json_encode(get_term_meta($termid), JSON_UNESCAPED_SLASHES)) . '"';
    }

    private static function chainId($chainId)
    {
        return "data-native-token=\"" . esc_attr($chainId) . "\"";
    }

    private static function listingId($marketplaceId)
    {
        return "data-listing-id=\"" . esc_attr($marketplaceId) . "\"";
    }

    private static function quantity($quantity)
    {
        return "data-quantity=\"" . esc_attr($quantity) . "\"";
    }

    private static function currencyAddress($currencyAddress)
    {
        return "data-currency-address=\"" . esc_attr($currencyAddress) . "\"";
    }

    private static function postId($postId)
    {
        return "data-post-id=\"" . esc_attr($postId) . "\"";
    }
}
