<?php

namespace NFT_Marketplace_Core\Internals\Helpers;

use Brick\Math\BigDecimal;
use NFT_Marketplace_Core\Internals\ContractHelperThirdWeb;
use SWeb3\SWeb3;
use SWeb3\SWeb3_Contract;

/**
 * Helper class to easily access all useful functions.
 *
 * @package   NFT Marketplace Core
 * @author    ModelTheme <support@modeltheme.com>
 * @copyright Copyright (C) 2012-2022, ModelTheme, support@modeltheme.com
 * @license   GPL-3.0
 * @link      https://modeltheme.com
 */
class Helper
{
	public static function contractHelperFromPostId($postID, &$additionalData = null)
	{
		$blockchains_term = get_the_terms($postID, 'nft_listing_blockchains');

		if (empty($blockchains_term)) {
			return Error::throw("10001", "NFT Does not have a blockchain appointed.", [
				"postId" => $postID
			]);
		}

		$termId = $blockchains_term[0]->term_id;

		$address = get_post_meta($postID, 'address', true);
		$nft_marketplace_core_default_contract_blockchain = get_post_meta($postID, 'nft_marketplace_core_default_contract_blockchain', true);

		global $wpdb;

		$blockchains_term_meta = get_term_meta($termId);

		$defaultContracts = get_option("nft_marketplace_core_default_contract_blockchain");

		if ($nft_marketplace_core_default_contract_blockchain) {
			$marketplace_blockchain_address_query =
				$wpdb->get_results("SELECT contract_address, contract_type,listing_price FROM " . $wpdb->prefix . "nft_marketplace_core_contracts WHERE contract_address = '" . esc_sql($nft_marketplace_core_default_contract_blockchain) . "'")[0];
		} elseif ($defaultContracts !== false && isset($defaultContracts[$termId])) {
			$marketplace_blockchain_address_query =
				$wpdb->get_results("SELECT contract_address, contract_type,listing_price FROM " . $wpdb->prefix . "nft_marketplace_core_contracts WHERE id = " . esc_sql($defaultContracts[$termId]))[0];
		} else {
			$marketplace_blockchain_address_query =
				$wpdb->get_results("SELECT contract_address, contract_type,listing_price FROM " . $wpdb->prefix . "nft_marketplace_core_contracts WHERE taxonomy_blockchain_id = " . esc_sql($termId))[0];
		}

		if ($marketplace_blockchain_address_query === null) {
			return Error::throw("10002", "Blockchain does not have a contract deployed.", [
				"postId" => $postID,
				"blockchainId" => $termId
			]);
		}

		$marketplace_blockchain_address = $marketplace_blockchain_address_query->contract_address;
		$provider = $blockchains_term_meta["nft_marketplace_core_taxonomy_blockchain_currency_rpc_url"][0];

		$additionalData = [
			"provider" => $provider,
			"term_id" => $termId,
			"marketplace_address" => $marketplace_blockchain_address,
			"listing_address" => $address,
			"currency_symbol" => $blockchains_term_meta["nft_marketplace_core_taxonomy_blockchain_currency_symbol"][0],
			"contract_type" => $marketplace_blockchain_address_query->contract_type,
			"marketplace_fee" => $marketplace_blockchain_address_query->listing_price,
			"chain_id" => hexdec(get_term_meta($termId, "nft_marketplace_core_taxonomy_blockchain_currency_chainid", true))
		];

		return new ContractHelperThirdWeb($provider, $marketplace_blockchain_address, $address);
	}

	public static function getCurrencyByContractAddress(string $address)
	{
		return get_terms([
			'taxonomy' => "nft-listing-currency",
			'hide_empty' => false,
			'meta_query' => array(
				array(
					'key' => 'nft_marketplace_core_taxonomy_currency_contract',
					'value' => $address,
					'compare' => '='
				)
			)
		]);
	}

	public static function areAddressesEqual($address1, $address2): bool
	{
		if (str_contains($address1, "0x0")) {
			$address1 = str_replace("0x0", "0x", $address1);
		}

		if (str_contains($address2, "0x0")) {
			$address2 = str_replace("0x0", "0x", $address2);
		}

		return strtolower($address1) === strtolower($address2);
	}

	public static function calcCollectionPropertyOccurrence($collectionId)
	{
		$the_query = new \WP_Query(array(
			'post_type' => 'nft-listing',
			'tax_query' => array(
				array(
					'taxonomy' => 'advert_tag',
					'field' => 'slug',
					'terms' => 'politics',
				)
			),
		));

		while ($the_query->have_posts()) :
			$the_query->the_post();
			// Show Posts ...
		endwhile;

		/* Restore original Post Data
		 * NB: Because we are using new WP_Query we aren't stomping on the
		 * original $wp_query and it does not need to be reset.
		*/
		wp_reset_postdata();
	}

	public static function getCurrencyDataByPostId($postID)
	{
		$customTokenAddress = get_post_meta($postID, "currency", true);
		$price = get_post_meta($postID, "currency_price", true);

		if ($customTokenAddress !== "0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee") {
			$currencyTermId = self::getCurrencyByContractAddress($customTokenAddress)->term_id;
			$currencyTermMeta = get_term_meta($currencyTermId, "nft_marketplace_core_taxonomy_currency_symbol", true);

			return [
				"token" => $customTokenAddress,
				"symbol" => $currencyTermMeta,
				"price" => Marketplace::formatCurrency($price),
				"image" => Assets::getCurrencyImageSrcByTermId($currencyTermId, [30, 30])
			];
		}

		$termData = get_the_terms($postID, "nft_listing_blockchains")[0];
		$currencyImage = get_term_meta($termData->term_id, "nft_marketplace_core_taxonomy_blockchain_currency_symbol", true);

		return [
			"token" => $customTokenAddress,
			"symbol" => $currencyImage,
			"image" => Assets::getCurrencyImageSrcByTermId($termData->term_id, [30, 30]),
			"price" => Marketplace::formatCurrency($price),
		];
	}

	public static function getCurrencyDisplayDataByPostId($postID)
	{
		$customTokenAddress = get_post_meta($postID, "nft_marketplace_core_nft_listing_currency", true);

		$price = get_post_meta($postID, "nft_marketplace_core_nft_listing_currency_price", true);
		if ($customTokenAddress !== "0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee" && $customTokenAddress !== "") {
			$currencyTermId = self::getCurrencyByContractAddress($customTokenAddress)[0]->term_id;
			$currencyTermMeta = get_term_meta($currencyTermId, "nft_marketplace_core_taxonomy_currency_symbol", true);
			echo Assets::getCurrencyImageSrcByTermId($currencyTermId, [30, 30]); ?>

			<?php if ($customTokenAddress === "0x596a61a4daa09afd4169a4ed75a5035fe6e3917e") { ?>
				<b class="nft-importer-price"><?php echo esc_html(BigDecimal::of($price)->exactlyDividedBy(BigDecimal::of(10)->power(6))) . ' ' . esc_html($currencyTermMeta); ?></b>
			<?php } else { ?>
				<b class="nft-importer-price"><?php echo esc_html(Marketplace::formatCurrency($price)) . ' ' . esc_html($currencyTermMeta); ?></b>
			<?php } ?>
			<?php
			return;
		}

		$termData = get_the_terms($postID, "nft_listing_blockchains")[0];
		$currencyImage = get_term_meta($termData->term_id, "nft_marketplace_core_taxonomy_blockchain_currency_symbol", true);
		echo Assets::getCurrencyImageSrcByTermId($termData->term_id, [30, 30]); ?>
		<b class="nft-importer-price"><?php echo esc_html(Marketplace::formatCurrency($price)) . ' ' . esc_html($currencyImage); ?></b>
		<?php
	}

	public static function getCurrency($price, $termID, $customTokenAddress = "0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee", $showIcon = true)
	{
		if ($customTokenAddress !== "0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee" && $customTokenAddress !== "") {
			$currencyTermId = self::getCurrencyByContractAddress($customTokenAddress)[0]->term_id;
			$currencyTermMeta = get_term_meta($currencyTermId, "nft_marketplace_core_taxonomy_currency_symbol", true);
			if ($showIcon) {
				echo Assets::getCurrencyImageSrcByTermId($currencyTermId, [30, 30]);
			} ?>
			<?php if ($customTokenAddress === "0x596a61a4daa09afd4169a4ed75a5035fe6e3917e") { ?>
				<b class="nft-importer-price"><?php echo esc_html(BigDecimal::of($price)->exactlyDividedBy(BigDecimal::of(10)->power(6))) . ' ' . esc_html($currencyTermMeta); ?></b>
			<?php } else { ?>
				<b class="nft-importer-price"><?php echo esc_html(Marketplace::formatCurrency($price)) . ' ' . esc_html($currencyTermMeta); ?></b>
			<?php } ?>
			<?php
			return;
		}
		if ($showIcon) {
			$currencyImage = get_term_meta($termID, "nft_marketplace_core_taxonomy_blockchain_currency_symbol", true);
			echo Assets::getCurrencyImageSrcByTermId($termID, [30, 30]);
		}
		?>
		<b class="nft-importer-price"><?php echo esc_html(Marketplace::formatCurrency($price)) . ' ' . esc_html($currencyImage); ?></b>
		<?php
	}

	public static function getCurrencyDisplayDataByPostIdRAW($price, $postID, $showIcon = true)
	{
		$customTokenAddress = get_post_meta($postID, "nft_marketplace_core_nft_listing_currency", true);

		if ($customTokenAddress !== "0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee" && $customTokenAddress !== "") {
			$currencyTermId = self::getCurrencyByContractAddress($customTokenAddress)[0]->term_id;
			$currencyTermMeta = get_term_meta($currencyTermId, "nft_marketplace_core_taxonomy_currency_symbol", true);
			if ($showIcon) {
				echo Assets::getCurrencyImageSrcByTermId($currencyTermId, [30, 30]);
			}
			echo esc_html(Marketplace::formatCurrency($price)) . ' ' . esc_html($currencyTermMeta);
			return;
		}

		$termData = get_the_terms($postID, "nft_listing_blockchains")[0];
		$currencyImage = get_term_meta($termData->term_id, "nft_marketplace_core_taxonomy_blockchain_currency_symbol", true);
		if ($showIcon) {

			echo Assets::getCurrencyImageSrcByTermId($termData->term_id, [30, 30]);
		}
		echo esc_html(Marketplace::formatCurrency($price)) . ' ' . esc_html($currencyImage);
	}

	/* Function to check whether the user who clicks the love button already loved the post */
	public
	static function isAlreadyLoved($post_id)
	{
		$user_id = get_current_user_id();
		$postmetadata_userIDs = get_post_meta($post_id, "nft_marketplace_core_user_loved", false);
		$users_loved = [];
		if (count($postmetadata_userIDs) != 0) {
			$users_loved = $postmetadata_userIDs[0];
		}
		if (!is_array($users_loved))
			$users_loved = [];
		if (in_array($user_id, $users_loved)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @throws \Exception
	 */
	public
	static function buildBlockchainConnection($nftAddress, $provider, $contractName, $sWeb3 = null)
	{
		$contractLocation = file_get_contents(plugin_dir_path(NFT_MARKETPLACE_CORE_PLUGIN_MAIN_FILE_LOCATION) . "contracts/abis/$contractName.json");

		if (!$contractLocation) throw new \Exception("Could not find contract abi!");

		return new SWeb3_contract($sWeb3 ?? new SWeb3($provider), $nftAddress, json_encode(json_decode($contractLocation, true)["abi"]));
	}

	public
	static function getUnknownContractType($nftAddress, $provider)
	{
		$addressToClass = apply_filters("nft_marketplace_core_append_unknown_contract_to_list_filter", [
			"ERC-721" => "Collection",
			"TokenERC721" => "Collection",
			"TokenERC1155" => "Edition",
			"DropERC1155" => "Edition"
		]);

		try {
			$contractCall = self::buildBlockchainConnection($nftAddress, $provider, "Unknown")->call("contractType");
			if (property_exists($contractCall, "error")) {
				return "Collection";
			}

			return $addressToClass[$contractCall->result];
		} catch (\Exception $e) {
			return $e->getMessage();
		}
	}

	/**
	 * cUrl to fetch metadata.
	 *
	 * @param $url
	 * @return bool|string
	 * @throws \Exception
	 * @since 3.2.0
	 */
	public
	static function fetchMetadata($url)
	{
		$tuCurl = curl_init();
		curl_setopt($tuCurl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($tuCurl, CURLOPT_URL, $url);
		curl_setopt($tuCurl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);

		curl_setopt($tuCurl, CURLOPT_HTTPHEADER, array("Content-Type: application/json"));

		//execute call
		$metadata = curl_exec($tuCurl);

		curl_close($tuCurl);

		if (curl_errno($tuCurl)) {
			throw new \Exception("Could not fetch metadata, $url CURL Error: ". curl_errno($tuCurl));
		}

		return $metadata;
	}
}
