<?php

namespace NFT_Marketplace_Core\Internals\Helpers;

use Brick\Math\BigDecimal;
use NFT_Marketplace_Core\Contracts\Factory;
use NFT_Marketplace_Core\Internals\Metas\BlockchainMeta;
use NFT_Marketplace_Core\Internals\Metas\NFTMeta;
use SWeb3\SWeb3;
use SWeb3\SWeb3_Contract;

class Marketplace
{
	/**
	 * Imports one NFT inside WordPress
	 *
	 * @param $address
	 * @param $tokenId
	 * @param $termId
	 * @param null $contract
	 * @return \WP_Error|array
	 * @throws \Exception
	 * @since 3.2.0
	 */
	public static function buildPostForNFT($address, $tokenId, $termId, $contract = null)
	{
		$provider = get_term_meta($termId, BlockchainMeta::rpc, true);

		// check if it already exists inside WordPress
//        if (self::isNftAlreadyImported($tokenId, $address)) {
//            return Error::throw("10003", esc_html__("NFT already exists.", NFT_MARKETPLACE_CORE_TEXTDOMAIN), [$tokenId, $address]);
//        }

		if (is_null($contract)) {
			// query for the NFT on the blockchain & determine what contract type it is
			$type = Helper::getUnknownContractType($address, $provider);

			try {
				$contract = Factory::getContractByType($type, $address, $provider);
			} catch (\Exception $e) {
				throw new \Exception(esc_html__("Could not determine what contract type is this.", NFT_MARKETPLACE_CORE_TEXTDOMAIN));
			}
		}

		try {
			$url = Assets::ipfsToURL($contract->getMetadataByTokenId($tokenId));
			if ($url === "") {
				throw new \Exception(esc_html__("Could not find any metadata on token no. $tokenId, is the token minted ?", NFT_MARKETPLACE_CORE_TEXTDOMAIN));
			}
		} catch (\Exception $e) {
			throw new \Exception(esc_html__("Could not retrieve metadata location.", NFT_MARKETPLACE_CORE_TEXTDOMAIN));
		}

		try {
			$metadata = Helper::fetchMetadata($url);
		} catch (\Exception $e) {
			throw new \Exception(esc_html__("Could not access IPFS.", NFT_MARKETPLACE_CORE_TEXTDOMAIN));
		}

		try {
			$metadata = \json_decode($metadata, true);
		} catch (\Exception $e) {
			throw new \Exception(esc_html__("Could not parse metadata.", NFT_MARKETPLACE_CORE_TEXTDOMAIN));
		}

		// fetch metadata and save it to WordPress
		return self::buildPost(
			$tokenId,
			$address,
			"",
			$metadata,
			$contract->getContractType(),
			get_term_by("term_id", $termId, "nft_listing_blockchains"),
			[$metadata["image"]]
		);
	}

	/**
	 * @param $address
	 * @param array|int $tokensToImport array to import specific tokens | INT to import from 0 to x number. | -1 to import everything
	 * @param $author
	 * @param $placeholderName
	 * @param $blockchainId
	 * @param null $collectionId
	 * @param null $listings
	 * @return array
	 * @throws \Exception
	 * @since 4.0.0
	 */
	public static function queueForImport($address, $tokensToImport, $author, $placeholderName, $blockchainId, $collectionId = null, $listings = null, $type = null, $bulkExisting = false)
	{
		global $wpdb;

		$createdPosts = [];

		// @todo handle $tokensToImport equals -1
		$provider = get_term_meta($blockchainId, BlockchainMeta::rpc, true);
		$contractType = Helper::getUnknownContractType($address, $provider);

		$_tokensToImport = is_int($tokensToImport) ? $tokensToImport : count($tokensToImport) - 1;

		if ($type === "bulk") {
			$_tokensToImport = (int)get_term_meta($collectionId, "nft_marketplace_core_taxonomy_collection_nfts", true);
		}

		// insert fake post(s)
		for ($token = ($bulkExisting ? ($_tokensToImport - $tokensToImport) : 0); $token < $_tokensToImport; $token++) {

			if (self::isNftAlreadyImported($token, $address)) {
				continue;
			}

			$table = $wpdb->prefix . 'nft_marketplace_core_queue';
			$alreadyExistsInQueue = ($wpdb->get_results("SELECT * FROM $table WHERE 'address' = $address AND 'token' = $token AND 'is_imported' = 0"));

			if ((is_array($alreadyExistsInQueue) && count($alreadyExistsInQueue) > 0) || is_null($alreadyExistsInQueue)) {
				continue;
			}

			$my_post = array(
				'post_title' => wp_strip_all_tags($placeholderName . " #$token"),
				'post_content' => " ",
				'post_status' => 'publish',
				'post_type' => 'nft-listing',
				'post_author' => $author,
				'meta_input' => array(
					NFTMeta::isImported => "no",
					NFTMeta::address => $address,
					NFTMeta::tokenId => $token,
				),
			);

			/** wp_insert_post with metadata  */
			$postId = wp_insert_post($my_post);
			$createdPosts[] = $postId;
			$blockchain = get_term($blockchainId, "nft_listing_blockchains");
			wp_set_object_terms($postId, $blockchain->slug, 'nft_listing_blockchains');

			if (!is_null($collectionId)) {
				$collection = get_term($collectionId, "nft-listing-collection");
				wp_set_object_terms($postId, $collection->slug, 'nft-listing-collection');
			}

			if ($listings !== null) {
				Marketplace::makeListing($postId, $listings[$token]);
			}

			$wpdb->insert($wpdb->prefix . 'nft_marketplace_core_queue', [
				"post_id" => $postId,
				"blockchain_id" => $blockchainId,
				"address" => $address,
				"token" => $token,
				"contract_type" => $contractType
			]);
		}

		return $createdPosts;
	}

	/**
	 * Imports all the NFTs inside a contract inside WordPress if they do not exist already
	 *
	 * @param $nftList
	 * @param $termId
	 * @return array
	 * @throws \Exception
	 * @since 3.2.0
	 */
	public static function runQueue($nftList, $termId)
	{
		global $wpdb;
		$posts = [];
		$provider = get_term_meta($termId, BlockchainMeta::rpc, true);
		$sweb3 = new SWeb3($provider);

		// get all metadatas.
		$sweb3->batch(true);

		foreach ($nftList as $nft) {
			try {
				$contract = Factory::getContractByType($nft["contract_type"], $nft["address"], $sweb3);
			} catch (\Exception $e) {
				$wpdb->insert($wpdb->prefix . 'nft_marketplace_core_queue_errors', [
					"error" => $e->getMessage(),
					"queue_item" => $nft["id"],
				]);
				continue;
			}

			$contract->getMetadataByTokenId($nft["token"]);
		}

		$requests = $sweb3->executeBatch();

		$sweb3->batch(false);
		foreach ($requests as $key => $request) {
			$nft = $nftList[$key];
			$contract = Factory::getContractByType($nft["contract_type"], $nft["address"], $sweb3);
			try {
				$metadata = Helper::fetchMetadata(Assets::ipfsToURL($contract->getBlockchainConnection()->DecodeData($contract::MetadataByTokenId, $request->result)->elem_1));
			} catch (\Exception $e) {
				$wpdb->insert($wpdb->prefix . 'nft_marketplace_core_queue_errors', [
					"error" => $e->getMessage(),
					"queue_item" => $nft["id"],
				]);
				continue;
			}

			try {
				$metadata = \json_decode($metadata, true);
			} catch (\Exception $e) {
				$wpdb->insert($wpdb->prefix . 'nft_marketplace_core_queue_errors', [
					"error" => $e->getMessage(),
					"queue_item" => $nft["id"],
				]);
				continue;
			}

			// fetch metadata and save it to WordPress
			$post = self::buildPost(
				$nft["token"],
				$nft["address"],
				$contract->getOwnerOf($nft["token"]),
				$metadata,
				$contract->getContractType(),
				get_term_by("term_id", $termId, "nft_listing_blockchains"),
				[$metadata["image"]]
			);

			$image_link = str_replace('${hash}', str_replace("ipfs://", "", $metadata["image"]), 'https://gateway.ipfs.io/ipfs/${hash}');
			$image_id = media_sideload_image($image_link, 0, '', 'id');

			set_post_thumbnail($post["ID"], $image_id);

			wp_update_post($post, true, false);
			$posts[] = $post;
			if (isset($nft["id"])) {
				$wpdb->update($wpdb->prefix . 'nft_marketplace_core_queue', array(
					'is_imported' => 1,
				), array(
					'id' => $nft["id"],
				));
			}
		}

		return $posts;
	}

	/**
	 * Check if an NFT already exists / Ignored if DEV Mode is on
	 *
	 * @param $tokenId
	 * @param $address
	 * @return bool
	 * @since 3.2.0
	 */
	private static function isNftAlreadyImported($tokenId, $address): bool
	{
		$args = array(
			'meta_query' => array(
				array(
					'key' => 'nft_marketplace_core_nft_listing_token_id',
					'value' => $tokenId
				),
				array(
					'key' => 'nft_marketplace_core_nft_listing_address',
					'value' => $address
				)
			),
			'post_type' => 'nft-listing',
			'posts_per_page' => -1,
		);

		$posts = get_posts($args);

		return !(count($posts) === 0 || NFT_MARKETPLACE_CORE_IS_DEVMODE_ON);
	}

	/**
	 * Save an NFT as a WordPress post.
	 *
	 * @param $tokenId
	 * @param $address
	 * @param $ownerAddress
	 * @param $metadata
	 * @param $contractType
	 * @param \WP_Term $blockchainTerm
	 * @param $images
	 * @return \WP_Error|array
	 * @since 3.2.0
	 */
	public static function buildPost($tokenId, $address, $ownerAddress, $metadata, $contractType, $blockchainTerm, $images)
	{
		global $wpdb;
//        if (self::isNftAlreadyImported($tokenId, $address)) {
//            return Error::throw("10003", esc_html__("NFT already exists.", NFT_MARKETPLACE_CORE_TEXTDOMAIN), [$tokenId, $address], false);
//        }

		$postID = self::getPostByNFTMetadata($tokenId, $address);

		$tablename = $wpdb->prefix . 'metamask_accounts';
		$user = $wpdb->get_row("SELECT * FROM  $tablename WHERE `account_address` = '$ownerAddress'", ARRAY_A);

		$my_post = array(
			'ID' => $postID,
			'post_title' => wp_strip_all_tags($metadata["name"]),
			'post_content' => $metadata["description"] ?? "",
			'post_status' => 'publish',
			'post_author' => is_null($user) ? 1 : $user['user_id'],
			'post_type' => "nft-listing",
			'comment_status' => "closed",
		);

		$my_post['meta_input'] = [
			"nft_marketplace_core_nft_listing_description_meta" => $metadata["description"] ?? "",
			"nft_marketplace_core_nft_listing_content_meta" => isset($metadata["unlockableContent"]),
			"nft_marketplace_core_nft_listing_content" => $metadata["unlockableContent"] ?? "",
			"nft_marketplace_core_nft_listing_explicit_meta" => $metadata["explicit"] ?? false,
			"nft_marketplace_core_nft_listing_owner" => $ownerAddress,
			"nft_marketplace_core_nft_listing_currency_price" => get_post_meta($postID, "nft_marketplace_core_nft_listing_currency_price", true) === "0" ? "0" : get_post_meta($postID, "nft_marketplace_core_nft_listing_currency_price", true),
			"nft_marketplace_core_nft_listing_address" => $address,
			"nft_marketplace_core_nft_listing_token_id" => $tokenId,
			"nft_marketplace_core_nft_listing_creator_user_id" => get_current_user_id(),
			"nft_marketplace_core_nft_listing_creator_address" => $ownerAddress,
			"nft_marketplace_core_nft_listing_contract_type" => $contractType,
			"nft_marketplace_core_nft_listing_deployed_blockchain_id" => get_term_meta($blockchainTerm->term_id, BlockchainMeta::chainid, true),
			NFTMeta::isImported => "yes"
		];

		if (isset($metadata["stats"])) {
			foreach ($metadata["stats"] as $stat) {
				if ($stat["name"] === "" || !isset($stat["value"]) || $stat["value"]["from"] === "" || !is_array($stat) || !is_array($stat["value"])) {
					break;
				}
				$my_post['meta_input']["nft_marketplace_core_nft_listing_group"][] = [
					"nft_marketplace_core_nft_listing_stat_name" => esc_html($stat["name"]),
					"nft_marketplace_core_nft_listing_stat_rate" => esc_html($stat["value"]["from"]),
				];
			}
		}

		if (isset($metadata["properties"])) {
			foreach ($metadata["properties"] as $properties) {
				if ($properties["name"] === "" || $properties["type"] === "" || !is_array($properties)) {
					break;
				}
				$my_post['meta_input']["nft_marketplace_core_nft_listing_properties_group"][] = [
					"nft_marketplace_core_nft_listing_property_name" => esc_html($properties["type"]),
					"nft_marketplace_core_nft_listing_property_value" => esc_html($properties["name"]),
				];
			}
		}

		if (isset($metadata["levels"])) {
			foreach ($metadata["levels"] as $levels) {
				if ($levels["name"] === "" || !isset($levels["value"]) || $levels["value"]["from"] === "" || !is_array($levels) || !is_array($levels["value"])) {
					break;
				}

				$my_post['meta_input']["nft_marketplace_core_nft_listing_level_group"][] = [
					"nft_marketplace_core_nft_listing_level_name" => esc_html($levels["name"]),
					"nft_marketplace_core_nft_listing_level_rate" => esc_html($levels["value"]["from"]),
				];
			}
		}

		$my_post['meta_input']["nft_marketplace_core_nft_listing_external_image"] = $images;

		return $my_post;
	}

	/**
	 * Marks the NFT as sold locally.
	 * @param $postID
	 * @return bool true if NFT is sold, false if it did not sell
	 * @throws \Exception
	 * @since 3.0.1
	 */
	public static function sellNFT($postID)
	{
		$contractHelper = Helper::contractHelperFromPostId($postID, $additionalData);
		$nft_marketplace_core_nft_marketplace_id = get_post_meta($postID, 'nft_marketplace_core_nft_marketplace_id', true);
		$item = $contractHelper->getMarketplaceItem($nft_marketplace_core_nft_marketplace_id);
		$quantity = $item->listing->quantity->toString();

		$savedQuantity = get_post_meta($postID, "nft_marketplace_core_nft_listing_quantity", true);


		if ($savedQuantity > $quantity) {
			update_post_meta($postID, "nft_marketplace_core_nft_listing_quantity", $quantity);

			$arg = array(
				'ID' => $postID,
				'post_author' => get_current_user_id(),
			);

			wp_update_post($arg);

			Provenience::save($postID, 3, [
				"username" => wp_get_current_user()->display_name,
				"price" => get_post_meta($postID, NFTMeta::price, true),
				"currency" => $additionalData["currency_symbol"],
			]);


			return true;
		}

		return false;
	}

	/**
	 * Cleans up all the offers for an NFT.
	 * @param $postId
	 * @return bool|int|\mysqli_result|resource|null
	 * @since 3.0.1
	 */
	public static function clearOffersOnNFT($postId)
	{
		global $wpdb;

		return $wpdb->delete($wpdb->prefix . "nft_marketplace_core_offers", ["nft_id" => $postId]);
	}

	/**
	 * Checks if the current NFT is on sale.
	 * @param $postID
	 * @return bool true if on sale, false if not on sale
	 * @since 3.0.1
	 */
	public static function isNFTOnSale($postID)
	{
		$nft_marketplace_core_nft_listing_start_time = get_post_meta($postID, 'nft_marketplace_core_nft_listing_start_time', true);
		$listingStarted = time() > $nft_marketplace_core_nft_listing_start_time;
		$hasItemLeftToSell = get_post_meta($postID, "nft_marketplace_core_nft_listing_quantity", true) > 0;
		$listingNotExpired = get_post_meta($postID, 'nft_marketplace_core_nft_listing_end_time', true) > time();

		return $listingStarted && $hasItemLeftToSell && $listingNotExpired;
	}

	public static function acceptOffer($postID, $offerId, $result)
	{
		$contractHelper = Helper::contractHelperFromPostId($postID, $additionalData);
		$offer = $contractHelper->getOffer($offerId);

		if (!$offer) {
			return;
		}
		$offer = $offer->offer;
		global $wpdb;
		$tablename = $wpdb->base_prefix.'metamask_accounts';

		$oferrer = $wpdb->get_row( "SELECT * FROM  $tablename WHERE `account_address` = '$offer->offeror'",ARRAY_A ) !== null;


		// Transfer the nft:
		$arg = array(
			'ID' => $postID,
			'post_author' => $oferrer
		);

		wp_update_post($arg);



		// Unlisted product
		self::cancelListing($postID);

		// Clear previous offers
		self::clearOffersOnNFT($postID);
	}

	/**
	 * Imports an offer.
	 * @param $postID
	 * @param $offerId
	 * @return void
	 * @throws \Exception
	 * @since 3.0.1
	 */
	public static function makeOfferOnNFT($postID, $offerId)
	{
		global $wpdb;

		$contractHelper = Helper::contractHelperFromPostId($postID, $additionalData);
		$nft_marketplace_core_nft_marketplace_id = get_post_meta($postID, 'nft_marketplace_core_nft_marketplace_id', true);

		$table = $wpdb->prefix . "nft_marketplace_core_offers";
		$offers = $wpdb->prepare("SELECT listingId FROM $table WHERE nft_id = %s", ["nft_id" => $postID]);
		$results = $wpdb->get_results($offers);
		// Product relisted, delete old offers!
		if (count($results) > 0 && $results[0]->listingId !== $nft_marketplace_core_nft_marketplace_id) {
			self::clearOffersOnNFT($postID);
		}

		$offer = $contractHelper->getOffer($offerId);
		$offer = $offer->offer;

		if ($offer->offerId->toString() === $offerId) {
			$offerJson =
				[
					"nft_id" => $postID,
					"listingId" => $offer->offerId->toString(),
					"offeror" => $offer->offeror,
					"quantityWanted" => $offer->quantity->toString(),
					"currency" => $offer->currency,
					"pricePerToken" => $offer->totalPrice->toString(),
					"expirationTimestamp" => $offer->expirationTimestamp->toString()
				];

			$wpdb->insert($wpdb->prefix . "nft_marketplace_core_offers", $offerJson);
		}
	}

	/**
	 * Cancels the listing locally.
	 * @param $postID
	 * @return void
	 * @since 3.0.1
	 */
	public static function cancelListing($postID)
	{
		Provenience::save($postID, 5, [
			"username" => wp_get_current_user()->display_name,
		]);

		delete_post_meta($postID, "nft_marketplace_core_nft_listing_currency_price");
		delete_post_meta($postID, "nft_marketplace_core_nft_listing_buyout_price_per_token");
		delete_post_meta($postID, "nft_marketplace_core_nft_listing_reserve_price_per_token");
		delete_post_meta($postID, "nft_marketplace_core_nft_listing_start_time");
		delete_post_meta($postID, "nft_marketplace_core_nft_listing_end_time");
		delete_post_meta($postID, "nft_marketplace_core_nft_listing_currency");
		delete_post_meta($postID, "nft_marketplace_core_nft_listing_quantity");
	}

	/**
	 * @throws \Exception
	 */
	public static function getPostByNFTMetadata($tokenId, $address)
	{
		$args = array(
			'meta_query' => array(
				array(
					'key' => 'nft_marketplace_core_nft_listing_token_id',
					'value' => $tokenId
				),
				array(
					'key' => 'nft_marketplace_core_nft_listing_address',
					'value' => $address
				)
			),
			'post_type' => 'nft-listing',
			'posts_per_page' => -1,
		);

		$posts = get_posts($args);

		if (!is_array($posts) || count($posts) === 0) {
			throw new \Exception(esc_html__("No post found for token id $tokenId and address $address", NFT_MARKETPLACE_CORE_TEXTDOMAIN));
		}

		return $posts[0]->ID;
	}

	/**
	 * Checks if the listing exists and saves it to WordPress
	 *
	 * @param $postID
	 * @param $marketplaceListingId
	 * @return void
	 * @throws \Exception
	 * @since 3.0.2
	 */
	public static function makeListing($postID, $marketplaceListingId)
	{
		$nft_marketplace_core_nft_listing_token_id = get_post_meta($postID, 'nft_marketplace_core_nft_listing_token_id', true);
		$nft_marketplace_core_nft_listing_address = get_post_meta($postID, 'nft_marketplace_core_nft_listing_address', true);

		$contractHelper = Helper::contractHelperFromPostId($postID, $additionalData);
		$nft = $contractHelper->getMarketplaceItem($marketplaceListingId);
		if ($nft->listing->listingId->toString() == $marketplaceListingId && Helper::areAddressesEqual($nft->listing->assetContract, $nft_marketplace_core_nft_listing_address) && $nft->listing->tokenId->toString() === $nft_marketplace_core_nft_listing_token_id) {
			update_post_meta($postID, "nft_marketplace_core_nft_marketplace_id", $marketplaceListingId);
			$listing = $nft->listing;

			Provenience::save($postID, 2, [
				"username" => wp_get_current_user()->display_name,
				"price" => $listing->pricePerToken->toString(),
				"currency" => $additionalData["currency_symbol"],
			]);

			update_post_meta($postID, "nft_marketplace_core_nft_listing_currency_price", $listing->pricePerToken->toString());
			update_post_meta($postID, "nft_marketplace_core_nft_listing_buyout_price_per_token", $listing->pricePerToken->toString());
			update_post_meta($postID, "nft_marketplace_core_nft_listing_reserve_price_per_token", $listing->pricePerToken->toString());
			update_post_meta($postID, "nft_marketplace_core_nft_listing_start_time", $listing->startTimestamp->toString());
			update_post_meta($postID, "nft_marketplace_core_nft_listing_end_time", $listing->endTimestamp->toString());
			update_post_meta($postID, "nft_marketplace_core_nft_listing_type", "direct");
			update_post_meta($postID, "nft_marketplace_core_nft_listing_currency", $listing->currency);
			update_post_meta($postID, "nft_marketplace_core_nft_listing_quantity", $listing->quantity->toString());
		} else {
			http_response_code(400);
			die();
		}
	}

	public static function formatCurrency($price)
	{
		$price = !empty($price) ? $price : 0;

		return BigDecimal::of($price)->exactlyDividedBy(BigDecimal::of(10)->power(18));
	}
}
