<?php
namespace NFT_Marketplace_Core\Rest;

use Brick\Math\BigDecimal;
use NFT_Marketplace_Core\Engine\Base;
use NFT_Marketplace_Core\Internals\Helpers\Assets;
use NFT_Marketplace_Core\Internals\Helpers\Helper;
use NFT_Marketplace_Core\Internals\Helpers\Marketplace;

/**
 * All Endpoints related to the vendor panel.
 *
 * @package   NFT Marketplace Core
 * @author    ModelTheme <support@modeltheme.com>
 * @copyright Copyright (C) 2012-2022, Modeltheme, support@modeltheme.com
 * @license   GPL-3.0
 * @link      https://modeltheme.com
 */
class UserPanel extends Base {

    /**
     * Initialize the class.
     *
     * @return void
     */
    public function initialize() {
        parent::initialize();

        \add_action( 'rest_api_init', array( $this, 'register_routes' ) );
    }

    public function register_routes() {
        register_rest_route(NFT_MARKETPLACE_CORE_TEXTDOMAIN, '/fetch-my-assets', array(
            'methods' => 'POST',
            'callback' => [$this, 'fetchMyAssets'],
            'permission_callback' => function ( $request ) {
                return is_user_logged_in();
            },
        ));

        register_rest_route(NFT_MARKETPLACE_CORE_TEXTDOMAIN, '/delete-nft', array(
            'methods' => 'DELETE',
            'callback' => [$this, 'deleteNFT'],
            'permission_callback' => function ( $request ) {
                return is_user_logged_in();
            },
        ));

        register_rest_route(NFT_MARKETPLACE_CORE_TEXTDOMAIN, '/delete-collection', array(
            'methods' => 'DELETE',
            'callback' => [$this, 'deleteCollection'],
            'permission_callback' => function ( $request ) {
                return is_user_logged_in();
            },
        ));

        register_rest_route(NFT_MARKETPLACE_CORE_TEXTDOMAIN, '/fetch-my-contracts', array(
            'methods' => 'POST',
            'callback' => [$this, 'fetchMyContracts'],
            'permission_callback' => function ( $request ) {
                return is_user_logged_in();
            },
        ));

        register_rest_route(NFT_MARKETPLACE_CORE_TEXTDOMAIN, '/fetch-my-collections', array(
            'methods' => 'POST',
            'callback' => [$this, 'fetchMyCollections'],
            'permission_callback' => function ( $request ) {
                return is_user_logged_in();
            },
        ));
    }


    public function fetchMyCollections($request) {

        $perPage = $request->get_param("length");
        $page = $request->get_param("start");

        $args = array(
            'hide_empty' => false, // also retrieve terms which are not used yet
            'meta_query' => array(
                array(
                    'key'       => 'nft_marketplace_core_taxonomy_collection_author',
                    'value'     => get_current_user_id(),
                    'compare'   => '='
                )
            ),
            'taxonomy'  => 'nft-listing-collection',
            'offset' => $page,
            'limit' => $perPage,
            'order' => 'DESC',
            'number' => $perPage
        );

        $terms = get_terms( $args );
        $finalData = [];
        foreach ($terms as $term) {
            $termMeta= get_term_meta($term->term_id);
            $image = $termMeta['collection-image-id'][0] !== null ? wp_get_attachment_image_src($termMeta['collection-image-id'][0]) : [Assets::getPlaceholderLocation()];

            if($termMeta['nft_marketplace_core_taxonomy_collection_author'][0] == get_current_user_id()) {
                $finalData[] = [
                    "id" => $term->term_id,
                    "collection_name" => $term->name,
                    "image" => $image,
                    "action_data" => [
                        "id" => $term->term_id,
                        "slug" => get_term_link($term->term_id)
                    ]
                ];
            }
        }
        unset($args['number'] );
        $all_terms = wp_count_terms( $args );

        $response = array(
            "draw" =>  $request->get_param("draw") + count($terms),
            "recordsTotal" =>   intval($all_terms),
            "recordsFiltered" =>  intval($all_terms),
            "data" => $finalData
        );

        return new \WP_REST_Response($response,200);
    }



    public function fetchMyAssets($request) {
        $perPage = $request->get_param("length");
        $page = $request->get_param("start");
        $draw = $request->get_param("draw");

        $args = [
            'post_type' => 'nft-listing',
            'post_status' => 'publish',
            'numberposts' => $perPage,
            'author' => get_current_user_id(),
            'offset' => $page,
            'limit' => $perPage,
            'order' => 'DESC',
            'number' => $perPage
        ];

        $posts = get_posts($args);

        $finalData = [];

        foreach ($posts as $post) {
			$meta = get_post_meta($post->ID);
			$nft_marketplace_core_nft_listing_blockchains_term = get_the_terms($post->ID, 'nft_listing_blockchains');
			$meta_term = get_term_meta($nft_marketplace_core_nft_listing_blockchains_term[0]->term_id);
			$price = !empty($meta["nft_marketplace_core_nft_listing_currency_price"][0]) ? $meta["nft_marketplace_core_nft_listing_currency_price"][0] : 0;
			$nft_marketplace_core_nft_listing_price = (float) get_post_meta($post->ID, 'nft_marketplace_core_nft_listing_currency_price', true);
			$customTokenAddress = get_post_meta($post->ID, "nft_marketplace_core_nft_listing_currency", true);
			if ($customTokenAddress !== "0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee" && $customTokenAddress !== "") {
				$price = BigDecimal::of($nft_marketplace_core_nft_listing_price)->exactlyDividedBy(BigDecimal::of(10)->power(6));
				$currencyTermId = Helper::getCurrencyByContractAddress($customTokenAddress)[0]->term_id;
				$currencyTermMeta = get_term_meta($currencyTermId, "nft_marketplace_core_taxonomy_currency_symbol", true);
			}
			$finalData[] = [
				"id" => $post->ID,
				"name" => $post->post_name,
				"price" => $customTokenAddress ?
					$price . " " . $currencyTermMeta
					: (
					!empty($meta["nft_marketplace_core_nft_listing_currency_price"][0]) ?
						(Marketplace::formatCurrency($price)) . " " . $meta_term["nft_marketplace_core_taxonomy_blockchain_currency_symbol"][0]
						: "Not listed"
					),
				"asset" => Assets::getNFTDisplayAsset($post->ID, "thumbnail"),
				"tokenid" => $meta["nft_marketplace_core_nft_listing_token_id"][0],
				"blockchain" => $nft_marketplace_core_nft_listing_blockchains_term[0]->name,
				"minted_nfts" => 1,
				"slug" => get_permalink($post->ID)
			];
        }

        $response = array(
            "draw" => $draw + count($posts),
            "recordsTotal" => intval(count_user_posts( get_current_user_id(), 'nft-listing')),
            "recordsFiltered" => intval(count_user_posts( get_current_user_id(), 'nft-listing')),
            "data" => $finalData
        );

        return new \WP_REST_Response($response,200);
    }

    public function deleteNFT($request) {
        $id = intval($_GET['id']);
        $post = get_post($id);

        if(intval($post->post_author) === get_current_user_id() && $post->post_type === "nft-listing") {
            wp_delete_post($id);

            return new \WP_REST_Response([
                "message" => "Deleted post with the name: ". $post->post_name
            ],204);
        }

        return new \WP_REST_Response([
            "message" => "Could not delete post with the name: ". $post->post_name
        ],405);
    }

    public function deleteCollection($request) {
        $id = intval($_GET['id']);

        $term = get_term($id);

        if(is_wp_error($term)) {
            return new \WP_REST_Response([
                "message" => "Could not delete collection with the name: ". $term->name
            ],405);
        }

        $deleteResult = wp_delete_term( $id, "nft-listing-collection", [
            'meta_query' => array(
                array(
                    'key'       => 'nft_marketplace_core_taxonomy_collection_author',
                    'value'     => get_current_user_id(),
                    'compare'   => '='
                )
            ),
        ]);

        if(!$deleteResult || is_wp_error($deleteResult)) {
            return new \WP_REST_Response([
                "message" => "Could not delete collection with the name: ". $term->name
            ],405);
        }

        return new \WP_REST_Response([
            "message" => "Deleted collection with the name: ". $term->name
        ],204);
    }


    public function fetchMyContracts($request) {
        $perPage = $request->get_param("length");
        $page = $request->get_param("start");
        $draw = $request->get_param("draw");

        global $wpdb;
        $key = "nft_marketplace_core_nft_listing_address";

        $statement = $wpdb->prepare( "
			SELECT DISTINCT pm.meta_value, p.guid, pm.post_id, p.post_author FROM $wpdb->postmeta pm
			LEFT JOIN $wpdb->posts p ON p.ID = pm.post_id
			WHERE pm.meta_key = '%s'
			AND p.post_status = 'publish'
			AND p.post_type = 'nft-listing'
			AND p.post_author = '%s'

			ORDER BY pm.meta_value 			 LIMIT %d OFFSET %d",
			[$key, wp_get_current_user()->ID, $perPage, $page]
        );

        $posts = $wpdb->get_results($statement);

        $finalData = [];

        foreach ($posts as $post) {
            $nft_marketplace_core_nft_listing_blockchains_term = get_the_terms($post->post_id, 'nft_listing_blockchains');
            $type = get_post_meta($post->post_id, "nft_marketplace_core_nft_listing_contract_type", true);
            if($post->post_author == wp_get_current_user()->ID) {
                $finalData[] = [
                    "id" => $post->post_id,
                    "address" => $post->meta_value,
                    "blockchain" => $nft_marketplace_core_nft_listing_blockchains_term[0]->name,
                    "type" => $type === "" ? "ERC-721" : $type,
                    "minted_nfts" => 1,
                    "action_data" => [
                        "id" => $post->post_id,
                        "minting" => get_term_meta($nft_marketplace_core_nft_listing_blockchains_term[0]->term_id)
                    ]
                ];
            }
        }

        $response = array(
            "draw" => $draw + count($posts),
            "recordsTotal" => intval(count_user_posts( get_current_user_id(), 'nft-listing')),
            "recordsFiltered" => intval(count_user_posts( get_current_user_id(), 'nft-listing')),
            "data" => $finalData
        );

        return new \WP_REST_Response($response,200);
    }
}
